// =============================================================================
// Header.cpp: Implementation of class cHeader
// Filter application, (c) lars.staurset@statkart.no 2003
// Some samples of headers, see separate file MessageSamples.txt
// =============================================================================

#include <iostream>
#include "Header.h"
#include "Exception.h"

// =========================================================================
// Public functions
// =========================================================================

// ---------------------------------------------------------------------------
// Class administration (construction, destruction, etc.)
// ---------------------------------------------------------------------------
// Constructor
cHeader::cHeader (const OutputLevel ceOutputLevel)
{
    // Store parameters
    m_eOutputLevel = ceOutputLevel;
    if (m_eOutputLevel >= OL_Debug)
        cout << "cHeader::cHeader\n";

    // Initialize private data members
    m_bMayAppendLine = false;
    m_lsLines.clear();
}

// Destructor
cHeader::~cHeader(void)
{
    if (m_eOutputLevel >= OL_Debug)
        cout << "cHeader::~cHeader\n";
}

// ---------------------------------------------------------------------------
// Data processing
// ---------------------------------------------------------------------------

// Process a header line
bool cHeader::ProcessLine (const string csLine)
{
    bool bLineToBeAppended = false;
    bool bMore = true;

    // Empty line signals end of header
    if (csLine.empty()) {
        bMore = false;

    // If line starts with space or tab, append it to previous line to facilitate analysis...
    } else if (csLine[0] == ' ' || csLine[0] == '\t') {
        if (m_bMayAppendLine)   // ...if receiving line permits
            bLineToBeAppended = true;

    // Normal line
    } else {
        // Will the current line accept the next to be appended?
        char sz[13];
        strcpy (sz, csLine.substr(0,12).data());
        // For some line types appending can result in very long lines, and they are not useful
        m_bMayAppendLine = strnicmp(sz, "To: ", 4) &&
                        strnicmp(sz, "Cc: ", 4) &&
                        strnicmp(sz, "Bcc: ", 5) &&
                        strnicmp(sz, "Received: ", 10) &&
                        strnicmp(sz, "References: ", 12);
    }


    // Append if conditions are met
    if (bLineToBeAppended) {
        if (m_eOutputLevel >= OL_Debug)
            cout << "cHeader::ProcessLine:Append \'" << csLine << "\'\n";
        list<string>::iterator iPos = m_lsLines.end(); // Now pointing beyond last entry
        (--iPos)->append(csLine);       // Append to last entry

    // Otherwise add line
    } else {
        if (m_eOutputLevel >= OL_Debug)
            cout << "cHeader::ProcessLine:Add \'" << csLine << "\'\n";
        m_lsLines.push_back (csLine);
    }
    return bMore;
}

// Save header to stream
void cHeader::Save (cOutStream *pOutfile)
{
    list<string>::const_iterator ciPos;
    for (ciPos = m_lsLines.begin(); ciPos != m_lsLines.end(); ++ciPos) {
        if (m_eOutputLevel >= OL_Debug)
            cout << "cHeader::Save: \'" << *ciPos << "\'\n";
        pOutfile->WriteLine(*ciPos);
    }
}

// =========================================================================
// Protected functions
// =========================================================================

// Does a string start with a specific string?
bool cHeader::StringStartsWith (const string csLine, const string csKey,
                                size_t &nNewStart) const
{
    if (m_eOutputLevel == OL_Dump)
        cout << "cHeader::StringStartsWith-i: csKey=\'" << csKey << "\' csLine=\'" << csLine << "\'\n";

    // Look for key at beginning of string
    bool bFound = csLine.substr (0, csKey.length()) == csKey;
    if (bFound) {
        nNewStart = csKey.length();
        if (m_eOutputLevel >= OL_Debug)
            cout << "cHeader::StringStartsWith: Found \'" << csKey << "\'\n";
    }

    if (m_eOutputLevel == OL_Dump)
        cout << "cHeader::StringStartsWith-o: bFound=" << bFound << "\n";
    return bFound;
}

// Find a key/value pair in a line
// Format: some text and then key="value" or key=value
size_t cHeader::Find (const string csLine, const string csKey,
                      const size_t nOldStart, size_t &nNewStart) const
{
    const char cszError[] = "Syntax error or unimplemented format\nCannot find %s in line \'%s\'\n";
    string sValue;
    size_t nStart = csLine.find (csKey, nOldStart);
    if (nStart == string::npos)
        return 0;
    else {
        // Key was found, look for "value" (often but not always delimited by quotation marks)
        nStart += csKey.length();   // Starting point for next find
        size_t nDelimiter1 = csLine.find ('\"', nStart);   // Position of first delimiter
        size_t nDelimiter2;
        if (nDelimiter1 == string::npos) {
            // No QM, look for '='
            nDelimiter1 = csLine.find ('=', nStart);
            if (nDelimiter1 == string::npos) {
                // No delimiter found
                throw cException (cszError, "leading \" or =", csLine);
            } else {
                // If value starts after =, it ends at end-of-line
                nDelimiter2 = csLine.length();
            }
        } else {
            nDelimiter2 = csLine.find ('\"', nDelimiter1 + 1); // Position of second QM
            if (nDelimiter2 == string::npos) {
                // No QM after value, let value end at end-of-line
                nDelimiter2 = csLine.length();
                if (m_eOutputLevel >= OL_Normal)
                    cout << "Format error: Trailing \" is missing in \'" << csLine << "\'\n";
            }
        }

        // Success, return starting point and length
        nNewStart = nDelimiter1 + 1;
        size_t nLength = nDelimiter2 - nDelimiter1 - 1;
        if (m_eOutputLevel >= OL_Debug)
            cout << "cHeader::Find: Found \'" << csKey << "\'=\'" << csLine.substr(nNewStart,nLength) << "\'\n";
        return nLength;
    }
}

// Find some value in a line
size_t cHeader::Find (const string csLine, const size_t nOldStart,
                      size_t &nNewStart) const
{
    // Look for space or tab before value
    size_t nDelimiter1 = csLine.find (' ', nOldStart);
    if (nDelimiter1 == string::npos)
        nDelimiter1 = csLine.find ('\t', nOldStart);
    if (nDelimiter1 == string::npos) {
        throw cException ("Syntax error or unimplemented format\nCannot find delimiter in line \'" + csLine + "\'\n");
    } else {

        // Look for space, tab, or end of line after value
        size_t nDelimiter2 = csLine.find (' ', nDelimiter1+1);
        if (nDelimiter2 == string::npos)
            nDelimiter2 = csLine.find ('\t', nDelimiter1+1);
        if (nDelimiter2 == string::npos)
            nDelimiter2 = csLine.length();

        // Success, determine starting point and length
        nNewStart = nDelimiter1 + 1;
        size_t nLength = nDelimiter2 - nDelimiter1 - 1;
        if (m_eOutputLevel >= OL_Debug)
            cout << "cHeader::Find: Found value \'" << csLine.substr(nNewStart,nLength) << "\'\n";
        return nLength;
    }
}

// =========================================================================
// Private functions
// =========================================================================
