// =============================================================================
// Mbox.cpp: Implementation of class cMbox
// Filter application, (c) lars.staurset@statkart.no 2003
// =============================================================================

#include <iostream>
#include "Mbox.h"
#include "Message.h"
#include "Exception.h"

// =========================================================================
// Public functions
// =========================================================================

// ---------------------------------------------------------------------------
// Class administration (construction, destruction, etc.)
// ---------------------------------------------------------------------------
// Constructor
cMbox::cMbox (const string csInStream, const string csOutStream,
              const bool cbAllowOverwrite, const OutputLevel ceOutputLevel)
: cCommand (csInStream, csOutStream, cbAllowOverwrite, ceOutputLevel)
{
    Output ("Removing attachments from \'" + csInStream + "\', new folder is \'" + csOutStream + "\'");
}

// Destructor
cMbox::~cMbox(void)
{
}

// ---------------------------------------------------------------------------
// Data processing
// ---------------------------------------------------------------------------
// Do the job
bool cMbox::Execute ()
{
    // Initialize variables
    int nMessageCount = 0;          // Message count
    int nDeletedMessageCount = 0;   // Count of deleted messages
    int nPartCount = 0;          // Total count of parts
    int nRemovedPartCount = 0;   // Total count of removed parts
    cMessage *pMessage = NULL;      // Message object
    string sLine;                   // Input line
    bool bException = false;        // true if a cException was thrown

    // Input loop
    while (ReadLine(&sLine, bException)) {

        // Start of message (is this criterion sufficient?)
        if (sLine.substr(0,7) == "From - ") {
            nMessageCount++;
            if (m_eOutputLevel >= OL_Normal)
                cout << "\nMessage no. " << nMessageCount << ":\n";

            // If a message is active, finish it...
            if (pMessage) {
                nPartCount += pMessage->GetPartCount();
                nRemovedPartCount += pMessage->GetRemovedPartCount();
                if (pMessage->GetDeleted())
                    nDeletedMessageCount++;
                delete pMessage;
            }

            // ...and start a new one
            pMessage = new cMessage (m_pOutStream, m_eOutputLevel);
        }

        // Send line to message object
        try {
            if (pMessage)
                pMessage->ProcessLine (sLine);
            else
                throw cException ("Input line out of order or unimplemented feature: \'" + sLine + "\'");
        } catch (const cException cE) {
            cout << cE.GetText() << "\n";
            bException = true;
            break;              // Terminate reading
        }
    }

    // If a message is active, finish it
    if (pMessage) {
        nPartCount += pMessage->GetPartCount();
        nRemovedPartCount += pMessage->GetRemovedPartCount();
        if (pMessage->GetDeleted())
            nDeletedMessageCount++;
        delete pMessage;
    }

    // Final report
    if (m_eOutputLevel >= OL_Normal)
        cout << endl;
    cout << nMessageCount << " messages were processed\n"
         << nPartCount << " attachments or similar parts were found, " << nRemovedPartCount << " removed\n";
    if (nDeletedMessageCount)
        cout << nDeletedMessageCount << " messages marked as deleted were removed (folder compacted)\n";
    if (m_eOutputLevel >= OL_Normal)
        cout << m_pInStream->GetLineCount() << " lines read, " << m_pOutStream->GetLineCount() << " written\n";
    cout << endl;
    if (bException) {
        string s;
        s = "An error occurred but nothing has happened to your mail in \'" + m_sInStream
          + "\'. You can safely delete \'" + m_sOutStream
          + "\'. You may also want to inform lars.staurset@statkart.no to help improve this program.";
        Output (s);
    } else if (nRemovedPartCount > 0 || nDeletedMessageCount > 0) {
        string s;
        s = "Your mail folder \'" + m_sInStream
          + "\' was processed and a new folder called \'" + m_sOutStream
          + "\' was created. Now please open \'" + m_sOutStream
          + "\' in your mail reader (restart it if necessary). Check carefully that all messages are readable and that the attachments have been removed.\n";
        Output (s);
        s = "To delete the attachments permanently you can - ON YOUR OWN RESPONSIBILITY - delete \'" + m_sInStream
          + "\' and rename \'" + m_sOutStream + "\' to \'" + m_sInStream
          + "\'. Please note that any message filters pointing to \'" + m_sInStream
          + "\' must be updated.\n";
        Output (s);
        s = "To undo all changes just delete \'" + m_sOutStream
          + "\'. Nothing has happened to \'" + m_sInStream + "\'.";
        Output (s);
    } else {
        Output ("Nothing significant has changed. You may delete the folder \'"
            + m_sOutStream + "\'.");
    }

    return !bException;
}

// =========================================================================
// Private functions
// =========================================================================

// Read a line
bool cMbox::ReadLine (string *ps, bool &bException)
{
    bool bMore = true;
    bException = false;
    try {
        bMore = m_pInStream->ReadLine(ps);
    } catch (const cException cE) {
        cout << cE.GetText() << "\n";
        bException = true;
        bMore = false;
    }
    return bMore;
}

// Split output string in max 79-character lines
void cMbox::Output (const string cs) const
{
    size_t nPos0 = 0;
    size_t nPos1 = 0;
    // Parse string for separators (space)
    for (size_t nPos = cs.find (' ', 0); nPos != string::npos;
                nPos = cs.find (' ', nPos+1)) {
        if (nPos - nPos0 > 79) {
            // Print a line
            cout << cs.substr (nPos0, nPos1 - nPos0) << endl;
            nPos0 = nPos1 + 1;
        } else {
            nPos1 = nPos;
        }
    }

    // Print last line
    if (cs.length() - nPos0 > 79) {
        cout << cs.substr (nPos0, nPos1 - nPos0) << endl;
        nPos0 = nPos1 + 1;
    }
    cout << cs.substr (nPos0, cs.length() - nPos0) << endl;
}