// =============================================================================
// Message.cpp: Implementation of class cMessage
// Filter application, (c) lars.staurset@statkart.no 2003
// =============================================================================

#include <iostream>
#include "Message.h"

// =========================================================================
// Public functions
// =========================================================================

// ---------------------------------------------------------------------------
// Class administration (construction, destruction, etc.)
// ---------------------------------------------------------------------------
// Constructor
cMessage::cMessage (cOutStream *pOutStream, const OutputLevel ceOutputLevel)
{
    // Store parameters
    m_pOutStream = pOutStream;
    m_eOutputLevel = ceOutputLevel;

    // Initialize private data members
    m_bDeleted = false;
    m_bInMessageHeader = true;
    m_sBoundary.erase();
    m_bFoundBoundary = false;
    m_eContentType = CT_Unknown;
    m_nPartCount = 0;
    m_nRemovedPartCount = 0;
    m_pMessageHeader = new cMessageHeader (m_eOutputLevel);
    m_pPart = NULL;
}

cMessage::~cMessage(void)
{
    // Clean-up in case of an exception
    // (normally the deletions will have happened elsewhere)
    if (m_pMessageHeader)
        delete m_pMessageHeader;
    if (m_pPart) {
        delete m_pPart;
    }
}

// ---------------------------------------------------------------------------
// Data processing
// ---------------------------------------------------------------------------
// Process a line from the input stream
void cMessage::ProcessLine (const string csLine)
{
    if (m_eOutputLevel >= OL_Debug)
        cout << "cMessage::ProcessLine-i: \'" << csLine << "\'\n";

    // If in message header, process it
    if (m_bInMessageHeader) {
        ProcessHeaderLine (csLine);

    // Look for final boundary
    } else if (csLine == "--" + m_sBoundary + "--") {
        ProcessFinalBoundary (csLine);

    // Is message marked as deleted?
    } else if (m_bDeleted) {
        if (m_eOutputLevel >= OL_Debug)
            cout << "cMessage::ProcessLine: Deleted message\n";

    // Message body to be processed - first look for parts boundary
    } else if (csLine == "--" + m_sBoundary) {
        ProcessPartBoundary (csLine);

    // If inside part, handle it
    } else if (m_pPart) {
        m_pPart->ProcessLine (csLine);

    // Something else: Outside part
    } else {
        if (m_eOutputLevel >= OL_Debug)
            cout << "cMessage::ProcessLine:In body or outside message: \'" << csLine << "\'\n";
        m_pOutStream->WriteLine (csLine);
    }

    if (m_eOutputLevel == OL_Dump)
        cout << "cMessage::ProcessLine-o\n";
}

// =========================================================================
// Private functions
// =========================================================================

// Process a message header line
void cMessage::ProcessHeaderLine (const string csLine)
{
    if (m_eOutputLevel >= OL_Debug)
        cout << "cMessage::ProcessHeaderLine-i\n";

    // Add line to temp header storage
    if (!m_pMessageHeader->ProcessLine (csLine)) {

        // Header ends here
        if (m_eOutputLevel >= OL_Debug)
            cout << "cMessage::ProcessHeaderLine::End of message header\n";
        m_bInMessageHeader = false;

        // Is message marked as deleted?
        m_bDeleted = m_pMessageHeader->Deleted();
        if (m_bDeleted) {
            if (m_eOutputLevel >= OL_Normal)
                cout << "Message was marked as deleted, output folder will be compacted\n";
        } else {
            // Not deleted
            m_pMessageHeader->Analyze();
            if (m_eOutputLevel >= OL_Debug)
                cout << "cMessage::ProcessHeaderLine::Saving message header\n";
            m_pMessageHeader->Save (m_pOutStream);
            m_eContentType = m_pMessageHeader->GetContentType();
            m_sBoundary = m_pMessageHeader->GetBoundary();
            if (m_eOutputLevel >= OL_Debug)
                cout << "cMessage::ProcessHeaderLine::Boundary=\'" << m_sBoundary << "\'\n";
            m_bFoundBoundary = false;
        }
    }
    if (m_eOutputLevel >= OL_Debug)
        cout << "cMessage::ProcessHeaderLine-o\n";
}

// Process final boundary (message terminator)
void cMessage::ProcessFinalBoundary (const string csLine)
{
    if (m_eOutputLevel >= OL_Debug)
        cout << "cMessage::ProcessFinalBoundary-i\n";

    // Final boundary found; finish any current part
    if (m_eOutputLevel >= OL_Debug)
        cout << "cMessage::ProcessLine: End of part no. " << m_nPartCount << endl;
    if (m_pPart) {
        if (m_pPart->GetRemoved())
            m_nRemovedPartCount++;
        delete m_pPart;
        m_pPart = NULL;
    }
    if (m_nRemovedPartCount == 0 && m_eOutputLevel >= OL_Debug)
        cout << "Content-Type suggested removable parts but none were found\n";

    // Write boundary to outstream
    m_pOutStream->WriteLine (csLine);

    if (m_eOutputLevel >= OL_Debug)
        cout << "cMessage::ProcessFinalBoundary-o\n";
}

// Process part boundary
void cMessage::ProcessPartBoundary (const string csLine)
{
    if (m_eOutputLevel >= OL_Debug)
        cout << "cMessage::ProcessPartBoundary-i\n";

    // Have we already found a boundary?
    if (m_bFoundBoundary) {

        // Yes, so this line starts part which is candidate for removal
        if (m_eOutputLevel >= OL_Debug)
            cout << "cMessage::ProcessLine-Found another boundary\n";

        // Finish any current part
        if (m_pPart) {
            if (m_pPart->GetRemoved())
                m_nRemovedPartCount++;
            delete m_pPart;
            m_pPart = NULL;
        }

        // Start a new part
        m_pPart = new cPart (m_eContentType, m_eOutputLevel, m_pOutStream);
        m_nPartCount++;

    } else {

        // First boundary signals message text, which passes through unprocessed
        if (m_eOutputLevel >= OL_Debug)
            cout << "cMessage::ProcessLine:Found first boundary\n";
        m_bFoundBoundary = true;
    }

    // Write boundary to outstream
    m_pOutStream->WriteLine (csLine);

    if (m_eOutputLevel >= OL_Debug)
        cout << "cMessage::ProcessPartBoundary-o\n";
}
